@@ -88,6 +88,79 @@ apollo_nl <- function(nl_settings, functionality){
   rows         = nl_settings[["rows"]]
   nlNests      = nl_settings[["nlNests"]]
   nlStructure  = nl_settings[["nlStructure"]]
+  nAlt <- length(V)
+  nObs <- tryCatch(nrow( get("apollo_inputs", parent.frame(), inherits=FALSE)$database ),
+                   error=function(e){
+                     lenV <- sapply(V, function(v) ifelse(is.array(v), dim(v)[1], length(v)) )
+                     lenA <- sapply(avail, function(v) ifelse(is.array(v), dim(v)[1], length(v)) )
+                     lenC <- length(choiceVar)
+                     return(max(lenV, lenA, lenC))
+                   })
+  altnames = names(alternatives)
+  altcodes = alternatives
+  nestnames= names(nlStructure)
+  
+  ### Format checks
+  # alternatives
+  test <- is.vector(alternatives) & !is.null(names(alternatives))
+  if(!test) stop("The \"alternatives\" argument needs to be a named vector")
+  # avail
+  test <- is.list(avail) || (length(avail)==1 && avail==1)
+  if(!test) stop("The \"avail\" argument needs to be a list or set to 1")
+  if(is.list(avail)){
+    lenA <- sapply(avail, function(v) ifelse(is.array(v), dim(v)[1], length(v)) )
+    test <- all(lenA==nObs | lenA==1)
+    if(!test) stop("All entries in \"avail\" need to be a scalar or a vector with one entry per observation in the \"database\"")
+  }
+  # choiceVar
+  test <- is.vector(choiceVar) && (length(choiceVar)==nObs || length(choiceVar)==1)
+  if(!test) stop("The \"choiceVar\" argument needs to be a scalar or a vector with one entry per observation in the \"database\"")
+  # V
+  if(!is.list(V)) stop("The \"V\" argument needs to be a list")
+  lenV <- sapply(V, function(v) ifelse(is.array(v), dim(v)[1], length(v)) )
+  test <- all(lenV==nObs | lenV==1) && !is.null(names(V))
+  if(!test) stop("Each element of \"V\" must be named and needs to be a scalar or a vector with one entry per observation in the \"database\"")  
+  # rows
+  test <- is.vector(rows) && ( (is.logical(rows) && length(rows)==nObs) || (length(rows)==1 && rows=="all") )
+  if(!test) stop("The \"rows\" argument needs to be \"all\" or a vector of boolean statements with one entry per observation in the \"database\"")
+  # functionality
+  test <- functionality %in% c("estimate","prediction","validate","zero_LL","conditionals","output","raw")
+  if(!test) stop("Non-permissable setting for \"functionality\"")
+  # nlStructure
+  test <- is.list(nlStructure) && !is.null(names(nlStructure)) && all(sapply(nlStructure, is.vector)) && all(sapply(nlStructure, is.character))
+  if(!test) stop("Argument \"nlStructure\" must be a named list of character vectors describing the content of each nest.")
+  
+  ### Expand availabilities if =1
+  avail_set <- FALSE
+  if(length(avail)==1 && avail==1){
+    avail <- as.list(setNames(rep(1,nAlt), altnames))
+    avail_set <- TRUE
+  } 
+  
+  ### Filter rows
+  if(length(rows)==1 && rows=="all") rows <- rep(TRUE, length(nObs))
+  if(any(!rows)){
+    avail <- lapply(avail, function(av) if(length(av)==1) return(av) else return(av[rows]))
+    choiceVar <- choiceVar[rows]
+    V <- lapply(V, apollo_keepRows, r=rows)
+    # update nObs
+    lenV <- sapply(V, function(v) ifelse(is.array(v), dim(v)[1], length(v)) )
+    lenA <- sapply(avail, function(v) ifelse(is.array(v), dim(v)[1], length(v)) )
+    lenC <- length(choiceVar)
+    nObs <- max(lenV, lenA, lenC)
+    rm(lenV, lenA, lenC)
+  }
+  
+  ### Reorder arguments
+  avail <- avail[altnames]
+  V     <- V[altnames]
+  
+  ### Set lambda_root to 1 if necessary
+  root_set  <- FALSE
+  if(!("root" %in% names(nlNests))){
+    root_set <- TRUE
+    nlNests["root"] <- 1
+  }
   
   # ############################## #
   #### functionality="validate" ####
@@ -97,43 +170,11 @@ apollo_nl <- function(nl_settings, functionality){
     # Store useful values
     apollo_control <- tryCatch(get("apollo_control", parent.frame(), inherits=FALSE),
                                error = function(e) list(noValidation=FALSE, noDiagnostics=FALSE))
-    nObs  <- length(choiceVar)
-    nAlts <- length(V)
-    avail_set <- FALSE
-    root_set  <- FALSE
-    altnames=names(alternatives)
-    altcodes=alternatives
-    nestnames <- names(nlStructure)
-    if(!("root" %in% names(nlNests))){
-      root_set <- TRUE
-      nlNests["root"] <- 1
-      #warning("Root lambda parameter set to 1.")
-    }
-    if(length(rows)==1 && rows=="all") rows=rep(TRUE,length(choiceVar))
-    choiceVar[!rows]=alternatives[1]
-    
-    # check rows statement
-    if(length(rows)!=length(choiceVar)) stop("The argument \"rows\" needs to either be \"all\" or a vector of length equal to the number of the rows in the data!")
-    
-    # Check that alternatives are named in altcodes and V
-    if(is.null(altnames) || is.null(altcodes) || is.null(names(V))) stop("Alternatives must be named, both in 'altnernatives' and 'V'.")
-    
-    # Create availability if necessary
-    if(!is.list(avail)){
-      avail_set <- TRUE
-      avail <- vector(mode="list", length=nAlts)
-      avail <- lapply(avail, function(a) 1)
-      names(avail) <- altnames
-      #warning("Full availability of alternatives assumed for NL component.")
-    }
-    # Reorder V to match altnames order, if necessary
-    if(any(altnames != names(V))) V <- V[altnames]
-    if(any(altnames != names(avail))) avail <- avail[altnames]
     
     if(apollo_control$noValidation==FALSE){
       
       # Check there are at least three alternatives
-      if(nAlts<3) stop("NL requires at least three alternatives")
+      if(nAlt<3) stop("NL requires at least three alternatives")
       
       # Check that choice vector is not empty
       if(nObs==0) stop("No choices to model")
@@ -147,17 +188,16 @@ apollo_nl <- function(nl_settings, functionality){
       if(!all(choiceLabs %in% altcodes)) stop("Value in choice column that is not included in altcodes.")
       
       # check that nothing unavailable is chosen
-      chosenunavail=0
-      j=1
-      while(j <= length(altnames)){
-        if(sum((choiceVar==altcodes[j])*(avail[[j]]==0)*rows)) chosenunavail=1
-        j=j+1
-      }
-      if(chosenunavail==1) stop("Some alternative(s) chosen despite being listed as unavailable\n")
+      for(j in 1:nAlt) if(any(choiceVar==altcodes[j] & avail[[j]]==0)) stop("Alternative ",altnames[j]," chosen despite being listed as unavailable\n")
       
       # check that all availabilities are either 0 or 1
       for(i in 1:length(avail)) if( !all(unique(avail[[i]]) %in% 0:1) ) stop("Some availability values are not 0 or 1.")
       
+      # Check that no available alternative has utility = NA
+      # Requires setting non available alternatives utility to 0 first
+      V <- mapply(function(v,a) apollo_setRows(v, !a, 0), V, avail, SIMPLIFY=FALSE)
+      if(any(sapply(V, anyNA))) warning("At least one utility contains one or more NA values at parameter starting values")
+      
       
       # checks that are specific to nlStructure component
       
@@ -195,52 +235,21 @@ apollo_nl <- function(nl_settings, functionality){
         if(sum(combined_elements==nestnames[j])!=1) stop("A defined nest needs to appear exactly once in a tree!")
         j=j+1
       }
-      
-      # reorder nlStructure if needed
-      nlStructure_ordered=list()
-      element_list="root"
-      j=1
-      while(j>0){
-        k=1
-        temp=rep(TRUE,length(element_list))
-        while(k<= length(element_list)){
-          if(element_list[k] %in% altnames) temp[k]=FALSE
-          k=k+1
-        }
-        element_list=element_list[temp]
-        if(length(element_list)>0){
-          nlStructure_ordered[[element_list[1]]]=nlStructure[[element_list[1]]]
-          element_list=c(element_list,nlStructure[[element_list[1]]])
-          element_list=element_list[-1]
-        }
-        j=length(element_list)
-      }
-      
-      nlStructure=nlStructure_ordered
-      
-      # confirm all checks are passed, print output and return TRUE
-      #cat("\nAll checks passed for NL model component\n")
     }
     if(apollo_control$noDiagnostics==FALSE){
       
-      #if(avail_set==TRUE) warning("Availability not provided to 'apollo_nl' (or some elements are NA). Full availability assumed.")
-      # turn scalar availabilities into vectors
+      ### turn scalar availabilities into vectors
       for(i in 1:length(avail)) if(length(avail[[i]])==1) avail[[i]] <- rep(avail[[i]], nObs)
       
-      # Construct summary table of availabilities and market share
-      availprint = colSums(rows*matrix(unlist(avail), ncol = length(avail))) # number of times each alt is available
-      choicematrix = matrix(0,nrow=4,ncol=length(altnames))
-      choicematrix[1,] = availprint
-      j=1
-      while(j<= length(altnames)){
-        choicematrix[2,j]=sum(choiceVar==altcodes[j] & rows) # number of times each alt is chosen
-        j=j+1
-      }
-      choicematrix[3,] = choicematrix[2,]/sum(rows)*100 # market share
+      ### Construct summary table of availabilities and market share
+      choicematrix = matrix(0, nrow=4, ncol=length(altnames), 
+                            dimnames=list(c("Times available", "Times chosen", "Percentage chosen overall",
+                                            "Percentage chosen when available"), altnames))
+      choicematrix[1,] = unlist(lapply(avail, sum))
+      for(j in 1:length(altnames)) choicematrix[2,j] = sum(choiceVar==altcodes[j]) # number of times each alt is chosen
+      choicematrix[3,] = choicematrix[2,]/nObs*100 # market share
       choicematrix[4,] = choicematrix[2,]/choicematrix[1,]*100 # market share controlled by availability
       choicematrix[4,!is.finite(choicematrix[4,])] <- 0
-      rownames(choicematrix) = c("Times available","Times chosen","Percentage chosen overall","Percentage chosen when available")
-      colnames(choicematrix) = altnames
       
       # Order tree structure
       nlStructure_ordered=list()
@@ -294,7 +303,6 @@ apollo_nl <- function(nl_settings, functionality){
               depth <- length(space)
               cat("\n",space,rep("-",3*(maxDepth-depth)),"-Alternative: ",nlStructure[[component]][j], sep="")
             } else {
-              #cat("\n", space, "-Nest: ", nlStructure[[component]][j], sep="")
               cat("\n",space,"-Nest: ",nlStructure[[component]][j]," (",round(nlNests[[nlStructure[[component]][j]]],4), ")", sep="")
               print_tree_level(nlStructure, nlStructure[[component]][j], preceding_nest_layer+1, space)
             }
@@ -303,7 +311,6 @@ apollo_nl <- function(nl_settings, functionality){
         }
         
         maxDepth <- max(sapply(ancestors, length))-1
-        #cat("\nNest: ",names(nlStructure)[[1]], sep="")
         cat("Nest: ",names(nlStructure)[[1]]," (",round(nlNests[[names(nlStructure)[[1]]]],4),")", sep="")
         
         print_tree_level(nlStructure, "root", preceding_nest_layer=0, space="|")
@@ -311,14 +318,6 @@ apollo_nl <- function(nl_settings, functionality){
       
       
       ### Printing diagnostics
-      #cat('Overview of choices for NL model component:\n')
-      #print(round(choicematrix,2))
-      #cat("\n")
-      #if(any(choicematrix[4,]==0)) cat("Warning: some alternatives are never chosen in your data!\n")
-      #if(any(choicematrix[4,]==1)) cat("Warning: some alternatives are always chosen when available!\n")
-      #cat("Nested logit structure:")
-      #print_tree(nlStructure, ancestors)
-      #cat("\n")
       content <- list(round(choicematrix,2))
       if(any(choicematrix[4,]==0)) content[[length(content)+1]] <- "Warning: some alternatives are never chosen in your data!"
       if(any(choicematrix[4,]==1)) content[[length(content)+1]] <- "Warning: some alternatives are always chosen when available!"
@@ -327,11 +326,6 @@ apollo_nl <- function(nl_settings, functionality){
       if(root_set==TRUE) content[[length(content)+1]] <- "Notice: Root lambda parameter set to 1."
       content[[length(content)+1]] <- "Nested structure:"
       content[[length(content)+1]] <- capture.output(print_tree(nlStructure, ancestors))
-      #content <- list(round(choicematrix,2),
-      #                ifelse(any(choicematrix[4,]==0), "Warning: some alternatives are never chosen in your data!", ""),
-      #                ifelse(any(choicematrix[4,]==1), "Warning: some alternatives are always chosen when available!", ""),
-      #                "Nested structure:",
-      #                capture.output(print_tree(nlStructure, ancestors)))
       apolloLog <- tryCatch(get("apollo_inputs", parent.frame(), inherits=TRUE )$apolloLog, error=function(e) return(NA))
       apollo_addLog("Overview of choices for NL model component:", content, apolloLog)
     }
@@ -344,30 +338,10 @@ apollo_nl <- function(nl_settings, functionality){
   # ############################## #
   
   if(functionality=="zero_LL"){
-    # Store useful values
-    nObs  <- length(choiceVar)
-    nAlts <- length(V)
-    avail_set <- FALSE
-    altnames=names(alternatives)
-    altcodes=alternatives
-    nestnames <- names(nlStructure)
-    if(!("root" %in% names(nlNests))) nlNests["root"] <- 1
-    if(length(rows)==1 && rows=="all") rows=rep(TRUE,length(choiceVar))
-    choiceVar[!rows]=alternatives[1]
-    
-    # Create availability if necessary
-    if(!is.list(avail)){
-      avail_set <- TRUE
-      avail <- vector(mode="list", length=nAlts)
-      avail <- lapply(avail, function(a) 1)
-      names(avail) <- altnames
-    }
-    
-    if(!anyNA(avail)) if(any(altnames != names(avail))) avail <- avail[altnames]
     for(i in 1:length(avail)) if(length(avail[[i]])==1) avail[[i]] <- rep(avail[[i]], nObs) # turn scalar availabilities into vectors
     nAvAlt <- rowSums(matrix(unlist(avail), ncol = length(avail))) # number of available alts in each observation
     P = 1/nAvAlt # likelihood at zero
-    P[!rows] <- 1
+    if(any(!rows)) P <- apollo_insertRows(P, rows, 1)
     return(P)
   }
   
@@ -377,29 +351,15 @@ apollo_nl <- function(nl_settings, functionality){
   
   if(functionality %in% c("estimate","prediction","conditionals","raw")){
     
-    # Store useful values
-    if(functionality=="raw" && length(choiceVar)==1 && is.na(choiceVar)) choiceVar = alternatives[1]
-    nObs  <- length(choiceVar)
-    nAlts <- length(V)
-    avail_set <- FALSE
-    altnames=names(alternatives)
-    altcodes=alternatives
-    nestnames <- names(nlStructure)
-    if(!("root" %in% names(nlNests))) nlNests["root"] <- 1
-    if(length(rows)==1 && rows=="all") rows=rep(TRUE,length(choiceVar))
-    choiceVar[!rows]=alternatives[1]
-    
-    # Create availability if necessary
-    if(!is.list(avail)){
-      avail_set <- TRUE
-      avail <- vector(mode="list", length=nAlts)
-      avail <- lapply(avail, function(a) 1)
-      names(avail) <- altnames
+    # Fix choiceVar if "raw" and choiceVar==NA
+    choiceNA = FALSE
+    if(length(choiceVar)==1 && is.na(choiceVar)){
+      choiceVar = alternatives[1]
+      choiceNA  = TRUE
     }
     
-    ### Reorder V and avail to match altnames order, if necessary
-    if(any(altnames != names(V))) V <- V[altnames]
-    if(!anyNA(avail)) if(any(altnames != names(avail))) avail <- avail[altnames]
+    # Set utility of unavailable alternatives to 0 to avoid numerical issues
+    V <- mapply(function(v,a) apollo_setRows(v, !a, 0), V, avail, SIMPLIFY=FALSE)
     
     combined_elements="root"
     j=1
@@ -431,7 +391,7 @@ apollo_nl <- function(nl_settings, functionality){
     
     # extract chosen utility
     chosenV <- Reduce('+',
-                      lapply(as.list(1:nAlts),
+                      lapply(as.list(1:nAlt),
                              FUN=function(i) (choiceVar==altcodes[i])*V[[altnames[i]]])
     )
     
@@ -495,26 +455,16 @@ apollo_nl <- function(nl_settings, functionality){
     
     # consider availabilities (it assumes Palts and avail are in the same order)
     Palts <- mapply('*', Palts, avail[1:length(altnames)], SIMPLIFY = FALSE)
+    Palts <- lapply(Palts, function(x) {
+      x[is.na(x)] <- 0
+      return(x)}) # replace all NaN by 0
     
-    if(functionality=="prediction"|(functionality=="raw")){
-      Palts <- lapply(Palts, function(x) {x[is.na(x)] <- 0
-      return(x)}) # replace all NaN by 0
-      # add an additional column with chosen
-      if(functionality=="prediction") Palts[["chosen"]] = Reduce('+', tmp <- mapply(function(x,y) (choiceVar==y)*x, Palts, as.list(altcodes), SIMPLIFY=FALSE) ) # keep only probability for chosen alternative
-      Palts <- lapply(Palts, function(p) {
-        # change likelihood of excluded columns for 1
-        if(is.vector(p)) p[!rows]  <- ifelse(functionality=="prediction",NA,1)
-        if(is.matrix(p)) p[!rows,] <- ifelse(functionality=="prediction",NA,1)
-        if(is.array(p) & length(dim(p))==3) p[!rows,,] <- ifelse(functionality=="prediction",NA,1)
-        return(p)
-      })
+    if(functionality=="prediction"| functionality=="raw"){
+      if(!choiceNA) Palts[["chosen"]] = Reduce('+', tmp <- mapply(function(x,y) (choiceVar==y)*x, Palts, as.list(altcodes), SIMPLIFY=FALSE) ) # keep only probability for chosen alternative
+      if(any(!rows)) Palts <- lapply(Palts, apollo_insertRows, r=rows, val=NA) # insert excluded rows with value NA
     } else {
-      Palts <- lapply(Palts, function(x) {x[is.na(x)] <- 0
-      return(x)}) # replace all NaN by 0
       Palts = Reduce('+', tmp <- mapply(function(x,y) (choiceVar==y)*x, Palts, as.list(altcodes), SIMPLIFY=FALSE) ) # keep only probability for chosen alternative
-      if(is.vector(Palts)) Palts[!rows]  <- 1
-      if(is.matrix(Palts)) Palts[!rows,] <- 1
-      if(is.array(Palts) && length(dim(Palts))==3) Palts[!rows,,] <- 1
+      if(any(!rows)) Palts <- apollo_insertRows(Palts, rows, 1) # insert excluded rows with value 1
     }
     
     return(Palts)
@@ -526,153 +476,22 @@ apollo_nl <- function(nl_settings, functionality){
   
   if(functionality=="output"){
     
-    # Store useful values
-    nObs  <- length(choiceVar)
-    nAlts <- length(V)
-    avail_set <- FALSE
-    root_set  <- FALSE
-    altnames=names(alternatives)
-    altcodes=alternatives
-    nestnames <- names(nlStructure)
-    if(!("root" %in% names(nlNests))){
-      root_set <- TRUE
-      nlNests["root"] <- 1
-    }
-    if(length(rows)==1 && rows=="all") rows=rep(TRUE,length(choiceVar))
-    choiceVar[!rows]=alternatives[1]
+    P <- apollo_nl(nl_settings, functionality="estimate")
     
-    # Create availability if necessary
-    if(!is.list(avail)){
-      avail_set <- TRUE
-      avail <- vector(mode="list", length=nAlts)
-      avail <- lapply(avail, function(a) 1)
-      names(avail) <- altnames
-    }
+    ### turn scalar availabilities into vectors
+    for(i in 1:length(avail)) if(length(avail[[i]])==1) avail[[i]] <- rep(avail[[i]], nObs)
     
-    ### Reorder V and avail to match altnames order, if necessary
-    if(any(altnames != names(V))) V <- V[altnames]
-    if(!anyNA(avail)) if(any(altnames != names(avail))) avail <- avail[altnames]
-    
-    combined_elements="root"
-    j=1
-    while(j<= length(nlStructure)){
-      combined_elements=c(combined_elements,nlStructure[[j]])
-      j=j+1
-    }
-    
-    # Order tree structure
-    nlStructure_ordered=list()
-    element_list="root"
-    j=1
-    while(j>0){
-      k=1
-      temp=rep(TRUE,length(element_list))
-      while(k<= length(element_list)){
-        if(element_list[k] %in% altnames) temp[k]=FALSE
-        k=k+1
-      }
-      element_list=element_list[temp]
-      if(length(element_list)>0){
-        nlStructure_ordered[[element_list[1]]]=nlStructure[[element_list[1]]]
-        element_list=c(element_list,nlStructure[[element_list[1]]])
-        element_list=element_list[-1]
-      }
-      j=length(element_list)
-    }
-    nlStructure=nlStructure_ordered
-    
-    # extract chosen utility
-    chosenV <- Reduce('+',
-                      lapply(as.list(1:nAlts),
-                             FUN=function(i) (choiceVar==altcodes[i])*V[[altnames[i]]])
-    )
-    
-    # substract chosen utility from all others for numerical stability
-    V = lapply(X=V, FUN=function(v) v-chosenV)
-    
-    # loop over nests to create new utility elements and new availability terms
-    k=length(nlStructure)
-    while(k>0){
-      nestK <- names(nlStructure)[k]
-      V[[nestK]] = 0
-      avail[[nestK]] = 1*( Reduce('+', avail[ nlStructure[[k]] ]) > 0 ) # calculate availability of nest
-      j = 1
-      while(j<= length(nlStructure[[k]])){
-        nodeJ <- nlStructure[[k]][j]
-        V[[nestK]] = V[[nestK]] + avail[[nodeJ]]*exp( V[[nodeJ]]/nlNests[[nestK]] )
-        j = j+1
-      }
-      V[[nestK]] = nlNests[[nestK]]*log(V[[nestK]])
-      k = k-1
-    }
-    
-    # work out ancestors
-    ancestors=list()
-    j=1
-    while(j<= length(altnames)){
-      altJ <- altnames[[j]]
-      ancestors[[altJ]] = altJ
-      current = altJ
-      k = length(nlStructure)
-      while(k>0){
-        if(current %in% nlStructure[[k]]){
-          ancestors[[altnames[[j]]]] = c(ancestors[[altJ]],names(nlStructure)[k])
-          current = names(nlStructure)[k]
-        }
-        k=k-1
-      }
-      j=j+1
-    }
-    
-    # calculate log(probabilities)
-    logPalts=list()
-    j=1
-    while(j <= length(altnames)){
-      logPalts[[j]]=0
-      k=1
-      ancestorsJ <- ancestors[[altnames[[j]]]]
-      while(k< length(ancestorsJ)){ # loop to level just below root
-        current_V = V[[ ancestorsJ[k] ]]
-        next_V    = V[[ ancestorsJ[k+1] ]]
-        logPalts[[j]] = logPalts[[j]] + (current_V-next_V)/nlNests[[ ancestorsJ[k+1] ]]
-        k=k+1
-      }
-      j=j+1
-    }
-    
-    Palts = lapply(X=logPalts, FUN=exp)
-    
-    tempnames=names(V)
-    names(Palts)=tempnames[1:length(altnames)]
-    
-    # consider availabilities (it assumes Palts and avail are in the same order)
-    Palts <- mapply('*', Palts, avail[1:length(altnames)], SIMPLIFY = FALSE)
-    
-    Palts <- lapply(Palts, function(x) {x[is.na(x)] <- 0
-    return(x)}) # replace all NaN by 0
-    P = Reduce('+', tmp <- mapply(function(x,y) (choiceVar==y)*x, Palts, as.list(altcodes), SIMPLIFY=FALSE) ) # keep only probability for chosen alternative
-    if(is.vector(P)) P[!rows]  <- 1
-    if(is.matrix(P)) P[!rows,] <- 1
-    if(is.array(P) && length(dim(P))==3) P[!rows,,,drop=FALSE] <- 1
-    
-    # Construct summary table of availabilities and market share
-    avail <- avail[altnames]
-    for(i in 1:length(avail)) if(length(avail[[i]])==1) avail[[i]] <- rep(avail[[i]], nObs) # turn scalar availabilities into vectors
-    availprint = colSums(rows*matrix(unlist(avail), ncol = length(avail))) # n times each alt is available
-    choicematrix = matrix(0,nrow=4,ncol=length(altnames))
-    choicematrix[1,] = availprint
-    j=1
-    while(j<= length(altnames)){
-      choicematrix[2,j]=sum(choiceVar==altcodes[j] & rows) # times chosen
-      j=j+1
-    }
-    choicematrix[3,] = choicematrix[2,]/sum(rows)*100 # market share
+    ### Construct summary table of availabilities and market share
+    choicematrix = matrix(0, nrow=4, ncol=length(altnames), 
+                          dimnames=list(c("Times available", "Times chosen", "Percentage chosen overall",
+                                          "Percentage chosen when available"), altnames))
+    choicematrix[1,] = unlist(lapply(avail, sum))
+    for(j in 1:length(altnames)) choicematrix[2,j] = sum(choiceVar==altcodes[j]) # number of times each alt is chosen
+    choicematrix[3,] = choicematrix[2,]/nObs*100 # market share
     choicematrix[4,] = choicematrix[2,]/choicematrix[1,]*100 # market share controlled by availability
     choicematrix[4,!is.finite(choicematrix[4,])] <- 0
-    rownames(choicematrix) = c("Times available","Times chosen","Percentage chosen overall","Percentage chosen when available")
-    colnames(choicematrix) = altnames
     
-    # Load function to write tree structure
+    ### Load function to write tree structure
     print_tree=function(nlStructure, ancestors){
       
       print_tree_level = function(nlStructure, component, preceding_nest_layer, space){
@@ -698,40 +517,46 @@ apollo_nl <- function(nl_settings, functionality){
       print_tree_level(nlStructure, "root", preceding_nest_layer=0, space="|")
     }
     
-    ## write diagnostics to a file named "modelName_tempOutput.txt" in a temporary directory.
-    #apollo_control <- tryCatch( get("apollo_inputs", parent.frame(), inherits=FALSE )$apollo_control,
-    #                            error=function(e){
-    #                              cat("apollo_nl could not retrieve apollo_control. No diagnostics in output.\n")
-    #                              return(NA)
-    #                            } )
-    #if(!(length(apollo_control)==1 && is.na(apollo_control))){
-    #  fileName <- paste(apollo_control$modelName, "_tempOutput.txt", sep="")
-    #  fileName <- file.path(tempdir(),fileName)
-    #  fileConn <- tryCatch( file(fileName, open="at"),
-    #                        error=function(e){
-    #                          cat('apollo_nl could not write diagnostics to temporary file. No diagnostics in output.\n')
-    #                          return(NA)
-    #                        })
-    #  if(!anyNA(fileConn)){
-    #    sink(fileConn)
-    #    on.exit({if(sink.number()>0) sink(); close(fileConn)})
-    #    if(apollo_control$noDiagnostics==FALSE){
-    #      cat('Overview of choices for NL model component:\n')
-    #      print(round(choicematrix,0))
-    #      cat("\n")
-    #    }
-    #    cat("Structure of nested logit component:")
-    #    print_tree(nlStructure, ancestors)
-    #    cat('\n')
-    #    if(sum(choicematrix[4,]==0)>0) cat("Warning: some alternatives are never chosen in your data!\n")
-    #  }
-    #}
+    ### Order tree structure
+    nlStructure_ordered=list()
+    element_list="root"
+    j=1
+    while(j>0){
+      k=1
+      temp=rep(TRUE,length(element_list))
+      while(k<= length(element_list)){
+        if(element_list[k] %in% altnames) temp[k]=FALSE
+        k=k+1
+      }
+      element_list=element_list[temp]
+      if(length(element_list)>0){
+        nlStructure_ordered[[element_list[1]]]=nlStructure[[element_list[1]]]
+        element_list=c(element_list,nlStructure[[element_list[1]]])
+        element_list=element_list[-1]
+      }
+      j=length(element_list)
+    }
+    nlStructure=nlStructure_ordered
     
-    #content <- list(round(choicematrix,2),
-    #                ifelse(any(choicematrix[4,]==0), "Warning: some alternatives are never chosen in your data!", ""),
-    #                ifelse(any(choicematrix[4,]==1), "Warning: some alternatives are always chosen when available!", ""),
-    #                "Final nesting structure:",
-    #                capture.output(print_tree(nlStructure, ancestors)))
+    # work out ancestors
+    ancestors=list()
+    j=1
+    while(j<= length(altnames)){
+      altJ <- altnames[[j]]
+      ancestors[[altJ]] = altJ
+      current = altJ
+      k = length(nlStructure)
+      while(k>0){
+        if(current %in% nlStructure[[k]]){
+          ancestors[[altnames[[j]]]] = c(ancestors[[altJ]],names(nlStructure)[k])
+          current = names(nlStructure)[k]
+        }
+        k=k-1
+      }
+      j=j+1
+    }
+    
+    ### Write to apollo log
     content <- list(round(choicematrix,2))
     if(any(choicematrix[4,]==0)) content[[length(content)+1]] <- "Warning: some alternatives are never chosen in your data!"
     if(any(choicematrix[4,]==1)) content[[length(content)+1]] <- "Warning: some alternatives are always chosen when available!"
